@@ -70,7 +70,7 @@ static size_t fwrite_le16(int i16, FILE *file)
    return fwrite(buf,2,1,file);
 }
 
-int write_wav_header(FILE *file, int rate, int mapping_family, int channels)
+int write_wav_header(FILE *file, int rate, int mapping_family, int channels, int fp)
 {
    int ret;
    int extensible;
@@ -79,17 +79,20 @@ int write_wav_header(FILE *file, int rate, int mapping_family, int channels)
       proper channel meanings. */
    extensible = mapping_family == 1 && 3 <= channels && channels <= 8;
 
+   /* >16 bit audio also requires WAVEFORMATEXTENSIBLE. */
+   extensible |= fp;
+
    ret = fprintf (file, "RIFF") >= 0;
    ret &= fwrite_le32 (0x7fffffff, file);
 
    ret &= fprintf (file, "WAVEfmt ") >= 0;
    ret &= fwrite_le32 (extensible ? 40 : 16, file);
-   ret &= fwrite_le16 (extensible ? 0xfffe : 1, file);
+   ret &= fwrite_le16 (extensible ? 0xfffe : (fp?3:1), file);
    ret &= fwrite_le16 (channels, file);
    ret &= fwrite_le32 (rate, file);
-   ret &= fwrite_le32 (2*channels*rate, file);
-   ret &= fwrite_le16 (2*channels, file);
-   ret &= fwrite_le16 (16, file);
+   ret &= fwrite_le32 ((fp?4:2)*channels*rate, file);
+   ret &= fwrite_le16 ((fp?4:2)*channels, file);
+   ret &= fwrite_le16 (fp?32:16, file);
 
    if(extensible)
    {
@@ -101,6 +104,14 @@ int write_wav_header(FILE *file, int rate, int mapping_family, int channels)
         0x80, 0x00,
         0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71
       };
+      static const unsigned char ksdataformat_subtype_float[16]=
+      {
+        0x03, 0x00, 0x00, 0x00,
+        0x00, 0x00,
+        0x10, 0x00,
+        0x80, 0x00,
+        0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71
+      };
       static const int wav_channel_masks[8] =
       {
          1,                      /* 1.0 mono */
@@ -113,9 +124,14 @@ int write_wav_header(FILE *file, int rate, int mapping_family, int channels)
          1|2|4|8|16|32|512|1024, /* 7.1 */
       };
       ret &= fwrite_le16 (22, file);
-      ret &= fwrite_le16 (16, file);
+      ret &= fwrite_le16 (fp?32:16, file);
       ret &= fwrite_le32 (wav_channel_masks[channels-1], file);
-      ret &= fwrite (ksdataformat_subtype_pcm, 16, 1, file);
+      if (!fp)
+      {
+         ret &= fwrite (ksdataformat_subtype_pcm, 16, 1, file);
+      } else {
+         ret &= fwrite (ksdataformat_subtype_float, 16, 1, file);
+      }
    }
 
    ret &= fprintf (file, "data") >= 0;
